function [Theta] = Func_Eval_Theta_SysComp(X, B, alpha, lossType)  
% This function computes the systematic component using B and alpha.
% For ordinary matrix models, Theta is X * B + 1 * alpha', which is a
% matrix. In the trace model case (two subcases), Theta is always a vector.
if ~contains(lossType, 'tracemodel')
    % ordinary matrix model Y ~ X * B + 1 * alpha' (matrix)
    Theta = X * B + alpha';     
else
    % In this trace model case, Theta is a vector
    if size(X, 3) > 1
        % The most general trace model: y_i (scalar) ~ < X(:, :, i), B > + alpha_i
        Theta = reshape(sum(bsxfun(@times, X, B), [1, 2]), [], 1) + alpha; % Theta_i = < X(:, :, i), B > + alpha_i
    else
        % if size(X, 3) == 1, i.e., a two-dim design matrix, 
        % we assume a "sample-based trace model": y_i (scalar) ~ < X(i, :), B(i, :) > + alpha_i
        Theta = sum(X .* B, 2) + alpha; 
    end
end