function [B_full, alpha_full, C_full, F] = Func_FullmodelMLE(X, Y, estimateIntercept, lossType, F)
% Provide an estimate using the design(s) and response (and possibly F),
% with no further low rank.
if ~exist('F', 'var') || isempty(F)
	F = [];
end

if strcmp(lossType, 'gaussian')
    if ~isempty(F)
        X = X * F;
    end
    if estimateIntercept == 1
        Y_colmeans = mean(Y);
        Y = Y - Y_colmeans;
        X_colmeans = mean(X);
        X = X - X_colmeans;
        C_full = pinv(X' * X) * (X' * Y);
        alpha_full = Y_colmeans - X_colmeans * C_full;
    else
        C_full = pinv(X' * X) * (X' * Y);
        alpha_full = zeros(size(Y, 2), 1);
    end
    if ~isempty(F)
        B_full = F * C_full;
    else
        B_full = C_full;
    end

elseif strcmp(lossType, 'gaussian-tracemodel')
    if estimateIntercept ~= 0 
        warning('Not implemented for the trace model with an intercept.')
    end
    if size(X, 3) == 1 
        % "sample-based trace model"
        if isempty(F)
            error('F should be provided.')
        end
        C_full = zeros(size(F, 2), size(X, 2)); 
        for tInd = 1:size(F, 2)
            tempY = Y(find(F(:, tInd)));
            tempX = X(find(F(:, tInd)), :);
            C_full(tInd, :) = pinv(tempX' * tempX) * (tempX' * tempY);
        end
        B_full = F * C_full;
        alpha_full = zeros(size(Y, 1), 1);
    else
        % In this case, we still assume the three dimensional X is derived from
        % a "sample-based trace model"
        myF = zeros(size(X, 3), size(X, 1));
        Xs = cell(size(X, 1), 1);
        for tInd = 1:size(X, 3)
            tInd2 = find(sum(X(:, :, tInd).^2, 2) > 0);
            if numel(tInd2) ~= 1, error('Wrong input designs.'), end
            myF(tInd, tInd2) = 1;
            Xs{tInd2} = [Xs{tInd2}; X(tInd2, :, tInd)];
        end
        if ~isempty(F) && ~all(all(myF == F))
            warning('Provided F is not accurate.')
        end
        F = myF;
            
        C_full = zeros(size(F, 2), size(X, 2)); 
        for tInd = 1:size(F, 2)
            tempY = Y(find(F(:, tInd)));
            tempX = Xs{tInd};
            C_full(tInd, :) = pinv(tempX' * tempX) * (tempX' * tempY);
        end
        B_full = F * C_full;
        alpha_full = zeros(size(Y, 1), 1);
        
    end
else
    error('Not implemented')
end