function [X, Y, B_true, corrMat, valX, valY, tstX, tstY] = ...
    Func_GeneCRRRData(n, m, d, X_given, q, r, sigma, dataName, valN, tstN, B_true, rho_given)

    
if ~exist('B_true', 'var') || isempty(B_true)
    % Generate a default B_true, using the provided parameters
    error('Not implemented yet!')
    B_strength_default = 1;
    B_true = randn(q, r) * randn(r, m);
    B_true = [B_true; zeros(d - q, m)] * B_strength_default;
end


if strcmp(dataName, 'mynorm-rhoGiven')
    if ~exist('rho_given', 'var') || isempty(rho_given)
        disp('No rho provided; use the default value 0.5.')
        rho_given = 0.5;
    else
        ;
    end
end

if ~isempty(X_given)
    % X is already provided by the user
    if n ~= size(X_given, 1), error('Sample size n and X_given mismatch!'), end 
    X = X_given; 
    corrMat = [];
    Theta_true = X * B_true;
    Y = Theta_true + sigma * randn(n, m);
    if valN > 0
        if valN ~= n, disp('When using fixed design, the design matrix for validation must be the same as the design for training (and so valN=n)'), end
        valX = X;
        valN = n;
        valY = Theta_true + sigma * randn(valN, m);
    else
        valX = []; valY = [];
    end
    if tstN > 0
        if tstN ~= n, disp('When using fixed design, the design matrix for test must be the same as the design for training (and so tstN=n)'), end
        tstX = X;
        tstN = n;
        tstY = Theta_true + sigma * randn(tstN, m);
    else
        tstX = []; tstY = [];
    end
    
else
    % Generate a random design each time calling the function, according to dataname
    totN = n + valN + tstN;
    if strcmp(dataName(1:2), 'my')
        X = randn(totN, d);
        if strcmp(dataName, 'mynorm-iid')
            rho=0;  
            corrMat = rho .^ abs([1:d]' * ones(1, d) - ones(d, 1) * [1:d]);
            X = X * chol(corrMat);
        elseif strcmp(dataName, 'mynorm-highcorr')
            rho=.9;  
            corrMat = rho .^ abs([1:d]' * ones(1, d) - ones(d, 1) * [1:d]);
            X = X * chol(corrMat);
        elseif strcmp(dataName, 'mynorm-medcorr')
            rho=.5;  
            corrMat = rho .^ abs([1:d]' * ones(1, d) - ones(d, 1) * [1:d]);
            X = X * chol(corrMat);
        elseif strcmp(dataName, 'mynorm-rhoGiven')
            rho = rho_given;  
            corrMat = rho .^ abs([1:d]' * ones(1, d) - ones(d, 1) * [1:d]);
            X = X * chol(corrMat); 
        elseif strcmp(dataName, 'mynorm-mildcorr')
            rho=.1;  
            corrMat = rho .^ abs([1:d]' * ones(1, d) - ones(d, 1) * [1:d]);
            X = X * chol(corrMat);        
        end        
    end
    Y = X * B_true + sigma * randn(totN, m);
    tstX = X((n+valN+1):end, :); valX = X((n+1):(n+valN), :); X = X(1:n, :); 
    tstY = Y((n+valN+1):end, :); valY = Y((n+1):(n+valN), :); Y = Y(1:n, :); 
    % Note: If valN = 0, validation X and Y will be []; f tstN = 0, test X and Y will be []. 
end

end

