function [grad_B, Theta, grad_alpha] = Func_GradB(X, Y, lossType, estimateIntercept, B_cur, alpha_cur)
% This function calculates the gradient wrt the unknown coefficient matrix
% B. The output also includes the systematic component, and the gradient
% wrt the intercepts (a column vector)
if ~contains(lossType, 'tracemodel')
    % ordinary matrix model Y ~ X * B + Alpha (matrix)
    [Theta] = Func_Eval_Theta_SysComp(X, B_cur, alpha_cur, lossType); % Theta = X * B_cur + alpha_cur';      
    [~, G] = Func_Loss_Grad(Y, Theta, lossType, 1); % gradient wrt the systematic component
    GtOnes = G' * ones(size(X, 1), 1); % gradient wrt the intercepts (col vector)
    XtG = (X' * G); % gradient wrt B
else % in this trace model case, the gradient wrt to B (XtG) should be calculated differently
    if size(X, 3) > 1
        % The most general trace model: y_i (scalar) ~ < X(:, :, i), B > + alpha_i
        [Theta] = Func_Eval_Theta_SysComp(X, B_cur, alpha_cur, lossType); % just a vector (assuming Y is a vector)
        [~, G] = Func_Loss_Grad(Y, Theta, lossType, 1);   % grad wrt to the systematic component, a vector in this case     
        XtG = sum(bsxfun(@times, X, reshape(G, 1, 1, [])), 3);    % grad wrt B: sum_i G(i) * X(:, :, i)
        GtOnes = G; % gradient wrt the intercept column vector
    else % if size(X, 3) == 1, i.e., a two-dim design matrix, we assume a sample-based model: y_i (scalar) ~ < X(i, :), B(i, :) > + alpha_i
        [Theta] = Func_Eval_Theta_SysComp(X, B_cur, alpha_cur, lossType); % Theta = sum(X .* B_cur, 2) + alpha_cur;
        [~, G] = Func_Loss_Grad(Y, Theta, lossType, 1); % gradient wrt the systematic component
        XtG = bsxfun(@times, X, G); % same as diag(G) * X; when using a later version of Matlab
        GtOnes = G; % gradient wrt the intercept column vector

    end
end
if ~all(size(XtG) == size(B_cur)) || ~all(size(GtOnes) == size(alpha_cur)) , error('Error in grad calculation'), end
grad_B = XtG; grad_alpha = GtOnes;    
end