function [B_ref, alpha_ref, S_ref, V_ref, F_ref, mu_ref] = ...
    Func_RoughClusterB_aux(X, Y, B, q, r,  lossType, estimateIntercept, convCtrl )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Given a B estimate, not necessarily feasible (satisfying r(B)<=r, q(B)<=q),
% construct a new B that is *feasible*, as well as S, V, F, mu, and alpha.
% These reference quantities can be used in the initialization of CRL and
% a *stagewise* method (estimation + PCA + clustering). After getting F, use RRR to get the rest. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 1. Peform PCA on B at a lower rank, to get B_red (of size p x r_pca)
[~, D, V] = svd(B, 'econ');
r_B = sum(diag(D) > 1e-5); 
r_pca = min(r_B, q); %r); % This is the dimension of a reduced B to do clustering
V_red = V(:, 1:r_pca);
B_red = B * V_red;

% 3. Run K-means on B_red, to extract F
[F, ~ ] = Func_kmeans_aux(B_red, q, B_red(1:q,:));

% 4. Given F, Call RRL to get mu, V, alpha, and construct B and S
[B_ref, alpha_ref, S_ref, V_ref, F_ref, mu_ref] ...
    = Func_Calibration(X, Y, lossType, estimateIntercept, convCtrl, r, F);

end

 
