clc; clear;
% rng(0);
rng(2);
addpath(genpath(pwd));

%% Load raw data
% Debug mode
debug = 0; % 1;
if debug > 0
    % Saving command window outputs
    timestamp = datetime('now');
    timestamp.Format = 'MM-dd-yyyy-HHmmss';
    diaryfile = ['CRL', char(timestamp)];
    diary(diaryfile);
    diary on;
end


tempfolder = './datasets/crab/'
Data = dlmread([tempfolder, 'crab.txt'], ' ', 1, 0);  
X_raw = [Data(:, 3), Data(:, 1)>=5]; % ]; %
X_raw = [ones(size(X_raw, 1), 1), X_raw]; % X_raw = X_raw(:, [1, 2]);
Y_raw =  Data(:, 4); %Data(:, 4) > 0; %Y_raw = Data(:, 4) ; % log(1 + Data(:, 4)) ; % 

lossType = 'gaussian-tracemodel'; 

% % Data Example 2:
% n = 100; d = 3; rho = 0.1;
% corrMat = rho .^ abs([1:d]' * ones(1, d) - ones(d, 1) * [1:d]);
% X_raw = randn(n, d) * chol(corrMat);
% Y_raw = randn(n, 1) * 1;
% Y_raw(1:n/2) = Y_raw(1:n/2) + X_raw(1:n/2, :) * (rand(d, 1) * 100); % (ones(d,1) + 100); %
% Y_raw((n/2+1):end) = Y_raw((n/2+1):end) + X_raw((n/2+1):end, :) * (rand(d, 1) * 100); % (ones(d,1) - 100); %
% lossType = 'gaussian-tracemodel';

%-------Intercept inclusion--------%
estimateIntercept = 0; % 1; % 1: Include the intercept in the model fitting; 0: not


% Get the number of observations and features of the raw dataset
[numObs, numFeats] = size(X_raw); numResps = size(Y_raw, 2);

%% 


%----------Preprocess raw data-----------%
preprocess = struct('Y', 'none', 'X', 'scaling'); % 'none'); % 'normalization');
    % We scale X so that a gradient descent alg (such as RRL) can work better
[X, Y] = Func_preprocess(X_raw, Y_raw, preprocess);
% X = X/sqrt(numObs); % this is simply to make the plot more clear


%----------Control parameters specification for algorithm------------%


%% CRL algorithm parameters
% // (q, r) grid specification 
%   Cluster-number grid: User can specify either a specific grid (vector)
%   for the cluster number or the # of grid points. If both are empty, the default values will be used
qGridInfo = struct('qGrid', [], 'qPoints', []);
qGridInfo.qGrid = [1:5]; % [1:5:numFeats];  % [clustnoOfInterest]; % grid
qGridInfo.qPoints = [];% 10;  % Number of grid points for cluster number (unused if qGrid is specified) 
%   Rank grid: User can specify either a specific grid (vector) for the rank
%   or the # of grid points. If both are empty, the default values will be used. 
rGridInfo = struct('rGrid', [], 'rPoints', []);
rGridInfo.rGrid = inf; %[]% % rGridInfo=inf: consider no models having lower rank than q. Alternatively, rGrid=[] means we can include models of further lower ranks; the result (opt+tuning) is the same on the crab data
rGridInfo.rPoints = [];% 5; % % Number of grid points (unused if rGrid is specified)

% // CRL initial values parameters: if 'rrr' or 'rand' is [], the program
%       will use the default way to fill the values in and generate corresponding initial values
numInits = struct('rrr', [], 'rand', [50]); % struct('rrr', [], 'rand', []); % 400

% // Multi-start parameters in CRL
multi_start_params = struct('NumsStarts', [], 'ItersForPrejudgement', []);
multi_start_params.NumsStarts = [-1, 3, 1]; %[-1, 1];        % number of initial vals used in prejudgement stages. //No matter what the 1st element is, in the first stage, all initial values will be used. Also, note that the last element is the number of starting values for the final optimization stage
multi_start_params.ItersForPrejudgement = [15, 15]; % [30];  %  ith entry: the number of iterations for the ith (prejudgement) stage. // If not enough elements are provided, we will repeat the last provided value to fill in the whole array

% // CRL convergence control parameters
convCtrl = struct('outer', [], 'inner', []);
convCtrl.outer = struct('maxiter', [], 'B_tol', []);
convCtrl.inner = struct('maxiter', [], 'B_tol', [], 'func_tol', [], 'F_tol', []);
%------outer loop stopping criterion------------%
convCtrl.outer.maxiter = 500; % 20, 50 % Maximum outer iteration allowed
convCtrl.outer.B_tol = 1e-05; % error tolerance in change of B (outer loop)
%------inner loop stopping criterion------------%
convCtrl.inner.maxiter = 10; % Maximum inner iteration allowed
convCtrl.inner.B_tol = 1e-03; % error tolerance in change of B (inner loop)
convCtrl.inner.func_tol = 1e-03; % error tolerance in change of function value
convCtrl.inner.F_tol = 0; % Using stability in membership matrix as convergence criterion

% // CRL line search criterion
linesearchCtrl.rho_redfactor = 0.5; % 0.25 % 0.75
linesearchCtrl.numRhosInTrial = 5; % 1: no search
linesearchCtrl.linesearchDir = 'adaptive';%'backward'; %  


if debug >0 
    % Display debugging messages
    % disp(['Number of prejudgement stages = ', num2str(numel(multi_start_params.NumsStarts)-1)]);
    disp(['No. of Starting points in prejudgement stages (first elem invalid) = ', num2str(multi_start_params.NumsStarts)]);
    disp(['Maximum iteration allowed for outer loop = ', num2str(convCtrl.outer.maxiter)]);
    disp(['Maximum iteration allowed for inner loop = ', num2str(convCtrl.inner.maxiter)]);
    if strcmp(lossType, 'gaussian') && ( (strcmp(preprocess.X, 'normalization') ||  strcmp(preprocess.X, 'centering')) && ...
        (strcmp(preprocess.Y, 'normalization') || strcmp(preprocess.Y, 'centering') || strcmp(preprocess.Y, 'whitening')) )
        disp('The intercept estimate is zero due to the centering of X & Y, and so estimateIntercept can be 0')
    end
    if ~isempty(qGridInfo.qGrid)
        disp(['The q-grid of desirable cluster numbers: ', num2str(qGridInfo.qGrid)]);
    elseif ~isempty(qGridInfo.qPoints)
        disp(['The number of grid points in the q-grid: ', num2str(qGridInfo.qPoints)]);
    else
        disp('A default q-grid for cluster numbers will be constructed');
    end
    if ~isempty(qGridInfo.qGrid)
        disp(['The r-grid of desirable ranks: ', num2str(rGridInfo.rGrid)]);
    elseif ~isempty(qGridInfo.qPoints)
        disp(['The number of grid points in the r-grid: ', num2str(rGridInfo.rPoints)]);
    else
        disp('A default r-grid for ranks will be constructed');
    end
    disp(['In constructing the startings points, the ranks of RRR initial values = ', num2str(numInits.rrr)]);
    disp(['In constructing the startings points, the number of random subset-sampling based initial values = ', num2str(numInits.rand)]);
end

%-----Obtaining CRL solution path---------%
tic
[Bs_CRL, alphas_CRL, Ss_CRL, Vs_CRL, Fs_CRL, mus_CRL, rankBs_CRL, clstnoBs_CRL, objvals_CRL] = ...    
    Func_CRL_SolPath(X, Y, qGridInfo, rGridInfo, numInits, lossType, estimateIntercept,  ...
                     multi_start_params, convCtrl, linesearchCtrl); 
% The obtained coefficients are for possibly scaled/center predictors--check struct preprocess
toc
clstnoBs_CRL
objvals_CRL %   216.6331
complforlogloss = (  1.5 * (clstnoBs_CRL .* size(Bs_CRL, 2))  + 1.1 * (size(Bs_CRL, 1) - clstnoBs_CRL) .* log(clstnoBs_CRL) ) / numel(Y) ; % Assume no further lower rank
complforlogloss(complforlogloss >= 1) = inf;
picvals = log(objvals_CRL) + complforlogloss;

optInd = find(picvals == min(picvals), 1, 'first'); 
disp(['q = ', num2str(clstnoBs_CRL(optInd))])

% if exist('tempfolder', 'var'), writematrix(Fs_CRL{optInd}, [tempfolder, 'Outoupt_F_CRL.txt']), end

