clc; clear;
rng(1342);
addpath(genpath(pwd));

%% Set up simulation data parameters
simuTimes = 20; % number of runs in the simulation
save_data = false; % true;
load_data = false; % true;
% X, y, noise parameters
n = 100; % 50 % number of observations for training
d = 50; % 100 % number of variables
m = 25; % 100 % number of responses
sigma = 1; % noise strength

dataName = 'mynorm-rhoGiven'; %'mynorm-mildcorr'; % 'mynorm-medcorr';
rho_given = 0.2;
fixDesign = false; %true; % if true: Fix the design matrix (in addition to B) whe generating a new dataset

tuningMethod = 'valData'; 
valN = 10000;  % sample size for validaiton dataset
tstN = 10000;  % sample size for test dataset
if fixDesign, valN = n; tstN = n; end
preprocess = struct('Y', 'none', 'X', 'none'); % struct('Y', 'normalization', 'X', 'normalization'); %  % The struct to preprocess data
% X options: 'none', 'centering', 'normalization'; 
% Y options: 'none', 'centering', 'normalization', 'whitening'


% B paramters (the true coefficient matrix)
%       Note: this matrix is only generated once and kept the same in different simulation runs
B_rank = 5; % rank of B_true without adding additional noise
B_clstno = 10; % number of distinct row pattern in B_true without adding additional noise
b_Strength = 1; %  signal strength factor
misspec_sigma = 0 %0.08 %  0.2; % 0.10; %  0.05; % the std error of additional elementwise noise to generate B_true
% Generate a true B matrix
B_prototype = randn(B_clstno, B_rank); % q rows, r columns
B_prototype(1, :) = zeros(1, B_rank); B_prototype(2:end, :) = B_prototype(2:end, :) + b_Strength * [1:(B_clstno-1)]'; % change the mean and include some sparsit
% B_true_left = -ones(d, B_rank);
% for rowInd = 1:d
%     B_true_left(rowInd, :) = B_prototype(mod(rowInd-1, B_clstno)+1, :);
% end
B_true_left = B_prototype(randsample(size(B_prototype, 1), d, true), :); % left factor
B_true_right = randn(B_rank, m);% + 1;
B_true = 1 *  B_true_left * B_true_right + misspec_sigma * randn(d, m);

% ranksOfInterest = B_rank + 1 * [-4:4]; % [B_rank]; % 
% ranksOfInterest = ranksOfInterest(ranksOfInterest > 0)
% clustnoOfInterest =  B_clstno + 1 * [-3:3]; % [B_clstno]; % 
% clustnoOfInterest = clustnoOfInterest(clustnoOfInterest > 0)


% // Loss function used for the learning problem
%   Use 'gaussian' for linear regression. Use 'bernoulli' for logistic
%   regression. Use 'poisson' for poisson regression. Use 'huber', 'tukey', 'lorenz' for robust regression
lossType = 'gaussian'; % Perform experiments in the regression setup
% // Intercept inclusion 
estimateIntercept = 0; % 1: include the intercept in the model fitting; 0: not

%% CRL algorithm parameters
% // (q, r) grid specification 
%   Cluster-number grid: User can specify either a specific grid (vector)
%   for the cluster number or the # of grid points. If both are empty, the default values will be used
qGridInfo = struct('qGrid', [], 'qPoints', []);
qGridInfo.qGrid = 5*[1:round(d/5)]; % [];  % [1:d] %   [clustnoOfInterest]; % grid
qGridInfo.qPoints = [];% 10;  % Number of grid points for cluster number (unused if qGrid is specified) 
%   Rank grid: User can specify either a specific grid (vector) for the rank
%   or the # of grid points. If both are empty, the default values will be used. 
rGridInfo = struct('rGrid', [], 'rPoints', []);
rGridInfo.rGrid = 5 * [1: round(min([n, d, m]/5))]; % []; %%[1: min([n, d, m])]; %  [ranksOfInterest]; % grid
rGridInfo.rPoints =  [];% 5; % % Number of grid points (unused if rGrid is specified)

% // CRL initial values: two types, one based on rrr, and the other based on random subset sampling
%    If 'rrr' or 'rand' is [], the program will use the default. 
%    If 'rrr' == [-1], no rrr-based initial values; if 'rand' = [0], no rand-type initial values
numInits = struct('rrr', [-1], 'rand', [5]); % struct('rrr', [], 'rand', []);

% // Multi-start parameters in CRL
multi_start_params = struct('NumsStarts', [], 'ItersForPrejudgement', []);
multi_start_params.NumsStarts = [-1, 3, 1]; % number of initial vals used in prejudgement stages. //No matter what the 1st element is, in the first stage, all initial values will be used. Also, note that the last element is the number of starting values for the final optimization stage
multi_start_params.ItersForPrejudgement = [3, 3]; % ith entry: the number of iterations for the ith (prejudgement) stage. // If not enough elements are provided, we will repeat the last provided value to fill in the whole array

% // CRL convergence control parameters
convCtrl = struct('outer', [], 'inner', []);
convCtrl.outer = struct('maxiter', [], 'B_tol', []);
convCtrl.inner = struct('maxiter', [], 'B_tol', [], 'func_tol', [], 'F_tol', []);
%------outer loop stopping criterion------------%
convCtrl.outer.maxiter = 15; % 20, 50 % Maximum outer iteration allowed
convCtrl.outer.B_tol = 1e-02; % error tolerance in change of B (outer loop)
%------inner loop stopping criterion------------%
convCtrl.inner.maxiter = 10; % Maximum inner iteration allowed
convCtrl.inner.B_tol = 1e-02; % error tolerance in change of B (inner loop)
convCtrl.inner.func_tol = 1e-02; % error tolerance in change of function value
convCtrl.inner.F_tol = 0; % Using stability in membership matrix as convergence criterion

% // CRL line search criterion
linesearchCtrl.rho_redfactor = 0.5; % 0.25 % 0.75
linesearchCtrl.numRhosInTrial = 5; % 1: no search
linesearchCtrl.linesearchDir = 'adaptive'; %'backward'; %   




% Display some messages
if strcmp(tuningMethod, 'valData') && valN <= 0
    error('Tuning method is validation error, but no validation samle-size is provided')
end

disp(['No. of Starting points in prejudgement stages (first elem invalid) = ', num2str(multi_start_params.NumsStarts)]);
disp(['Maximum iteration allowed for outer loop = ', num2str(convCtrl.outer.maxiter)]);
disp(['Maximum iteration allowed for inner loop = ', num2str(convCtrl.inner.maxiter)]);
if strcmp(lossType, 'gaussian') && ( (strcmp(preprocess.X, 'normalization') ||  strcmp(preprocess.X, 'centering')) && ...
    (strcmp(preprocess.Y, 'normalization') || strcmp(preprocess.Y, 'centering') || strcmp(preprocess.Y, 'whitening')) )
    disp('The intercept estimate is zero due to the centering of X & Y, and so estimateIntercept can be 0')
end
if ~isempty(qGridInfo.qGrid)
    disp(['The q-grid of desirable cluster numbers: ', num2str(qGridInfo.qGrid)]);
elseif ~isempty(qGridInfo.qPoints)
    disp(['The number of grid points in the q-grid: ', num2str(qGridInfo.qPoints)]);
else
    disp('A default q-grid for cluster numbers will be constructed');
end
if ~isempty(qGridInfo.qGrid)
    disp(['The r-grid of desirable ranks: ', num2str(rGridInfo.rGrid)]);
elseif ~isempty(qGridInfo.qPoints)
    disp(['The number of grid points in the r-grid: ', num2str(rGridInfo.rPoints)]);
else
    disp('A default r-grid for ranks will be constructed');
end
disp(['In constructing the startings points, the ranks of RRR initial values = ', num2str(numInits.rrr)]);
disp(['In constructing the startings points, the number of random subset-sampling based initial values = ', num2str(numInits.rand)]);


%% Data generation and run algorithms
for simuIndex = 1:simuTimes
    methodInd = 0;
    disp('===============================================================')
    disp(['Simulation index = ', num2str(simuIndex)])
    disp('===============================================================')
    rng(simuIndex); % ensure we generate the same dataset for the purpose of debugging 
    
    % The following experiments are for regression only
    if fixDesign && simuIndex > 1 
        % Fixed design: Use the same design as in the first dataset
        X_given  = X_raw;
        [X_raw, Y_raw, B_true, ~, valX_raw, valY_raw, tstX_raw, tstY_raw] = ...
            Func_GeneCRRRData(n, m, d, X_given, B_clstno, B_rank, sigma, dataName, valN, tstN, B_true, rho_given); 
    else
        % Generate a new random design following the same distribution
        X_given  = [];
        [X_raw, Y_raw, B_true, corrMat, valX_raw, valY_raw, tstX_raw, tstY_raw] = ...
            Func_GeneCRRRData(n, m, d, X_given, B_clstno, B_rank, sigma, dataName, valN, tstN, B_true, rho_given); 
    end
    
    valN = size(valX_raw, 1); tstN = size(tstX_raw, 1); 
    
    %----------Preprocess raw data-----------%
    [X, Y, tstX, tstY, valX, valY] = Func_preprocess(X_raw, Y_raw, preprocess, tstX_raw, tstY_raw, valX_raw, valY_raw);
        
    %----------Save data-----------%
    if save_data
        datafilename_X = sprintf('./datasets/simu_data/X%d.csv', simuIndex);
        datafilename_Y = sprintf('./datasets/simu_data/Y%d.csv', simuIndex);
        datafilename_B = sprintf('./datasets/simu_data/B_true.csv');
        dlmwrite(datafilename_X, X, 'precision', 7)
        dlmwrite(datafilename_Y, Y, 'precision', 7)
        dlmwrite(datafilename_B, B_true, 'precision', 7)
        if ~isempty(valX_raw)
            datafilename_valX = sprintf('./datasets/simu_data/valX%d.csv', simuIndex);
            datafilename_valY = sprintf('./datasets/simu_data/valY%d.csv', simuIndex);
            dlmwrite(datafilename_valX, valX, 'precision', 7)
            dlmwrite(datafilename_valY, valY, 'precision', 7)
        end
        if ~isempty(tstX_raw)
            datafilename_tstX = sprintf('./datasets/simu_data/tstX%d.csv', simuIndex);
            datafilename_tstY = sprintf('./datasets/simu_data/tstY%d.csv', simuIndex);
            dlmwrite(datafilename_tstX, tstX, 'precision', 7)
            dlmwrite(datafilename_tstY, tstY, 'precision', 7)
        end
        if ~isempty(corrMat)
            datafilename_corrMat = sprintf('./datasets/simu_data/corrMat.csv');
            dlmwrite(datafilename_corrMat, corrMat, 'precision', 7)
        end
    end
    
    %% Method: CRL
    % Obtain a solution path %
    if ~load_data
        tic
        [Bs_CRL, alphas_CRL, Ss_CRL, Vs_CRL, Fs_CRL, mus_CRL, rankBs_CRL, clstnoBs_CRL, objvals_CRL] = ...    
            Func_CRL_SolPath(X, Y, qGridInfo, rGridInfo, numInits, lossType, estimateIntercept,  ...
                             multi_start_params, convCtrl, linesearchCtrl);
        toc

        if save_data
            save(['./simures/', 'CRL_res_ind', num2str(simuIndex), '_missigma', num2str(misspec_sigma), '.mat'], 'Bs_CRL', 'alphas_CRL', 'Ss_CRL', 'Vs_CRL', 'Fs_CRL', 'mus_CRL', 'rankBs_CRL', 'clstnoBs_CRL', 'objvals_CRL');
        end
    else 
        load(['./simures/', 'CRL_res_ind', num2str(simuIndex), '_missigma', num2str(misspec_sigma), '.mat']);
    end
    
    % PIC
	picvals = objvals_CRL ./ (1 - (  1.8 * (clstnoBs_CRL + size(Bs_CRL, 2)) .* rankBs_CRL  + 0.1 * (size(Bs_CRL, 1) - clstnoBs_CRL) .* log(clstnoBs_CRL) ) / numel(Y) );
%     picvals = log(objvals_CRL) + (  1.5 * (clstnoBs_CRL + size(Bs_CRL, 2) - rankBs_CRL) .* rankBs_CRL  + .5 * (size(Bs_CRL, 1) - clstnoBs_CRL) .* log(clstnoBs_CRL) )  / numel(Y);
    picvals(picvals<0) = inf;
    picOptInd = find(picvals == min(picvals), 1, 'first'); 
    disp(['For CRL+PIC, the best estimate is obtained at r=', num2str(rankBs_CRL(picOptInd)), ', q=', num2str(clstnoBs_CRL(picOptInd)), '. picOptInd=', num2str(picOptInd)])
    
    % Select the best model according the validation error, and calc. its test error   
    [B_est, optInd, Berr, XBerr, tstErr] = validationAndTest_aux(Bs_CRL, tuningMethod, lossType, X, Y, valX, valY, tstX, tstY, B_true, fixDesign, corrMat);
    
    methodInd = methodInd + 1;
    Berrs(methodInd, simuIndex) = Berr;
    XBerrs(methodInd, simuIndex) = XBerr;
    tsterrs(methodInd, simuIndex) = tstErr;
    

    % Check some info
    B_CRL_opt = B_est; alpha_CRL_opt = alphas_CRL(:, optInd);
    S_CRL_opt = Ss_CRL{optInd}; V_CRL_opt = Vs_CRL{optInd}; F_CRL_opt = Fs_CRL{optInd}; mu_CRL_opt = mus_CRL{optInd}; 
    rankB_CRL_opt = rankBs_CRL(optInd); clstnoB_CRL_opt = clstnoBs_CRL(optInd);
    disp(['For CRL solution path, the best estimate is obtained at r=', num2str(rankB_CRL_opt), ', q=', num2str(clstnoB_CRL_opt), '. OptInd=', num2str(optInd)])
    CRLModelsizes(:, simuIndex) = [rankB_CRL_opt; clstnoB_CRL_opt];

%         %% Method: Group LASSO
%         % Obtain a solution path
%         probType = []; % We are not doing selective RRR on the linearized problem
%         regType = 'pen'; % Use penalized version of regularization to obtain solution path for lambda grid
%         full2d = 1; % We will directly get the solution path for lambda grid for the full rank model
%         % For group lasso there is no rank reduction. 
%         rankGiven = [min([n, d, m])];
%         rankGridUBnd = [];
%         % Providing lambda grid
%         lambdasGiven =[]; %Grid of lambda values provided by the user
%         nPoints = 50; % number of desired lambda values, only checked if grid is empty
%         % Convergence control
%         maxIT = []; % Not required for group lasso, only used in rank reduction step
%         grpSelMaxIT = -1; % Default -1
%         errBnd = 1e-4;
%         thresholdingWay = 'soft'; % Options 'soft', 'hard', 'prop', 'hybrid', 'hybrid-prop'
%         Nu = 0; % Shrinkage in group lasso. Not required.
%         rem_ctrl = struct('zeroRem', 1, 'complexRem', 1,'redundRem', 1); % Removing redundant solution points. Should be 1.
%         output = 0; %1; % Display option.
% 
%         [Bs_GLasso, rankBs_GLasso, grpBs_GLasso, gridLs_GLasso, nzPatts_GLasso, trnerrs_GLasso,...
%             numBs_GLasso, Ss_GLasso, Vs_GLasso] = Func_SRRRPathComp2(X,...
%                 Y, probType, regType, rankGiven, rankGridUBnd, lambdasGiven,...
%                     full2d, nPoints, maxIT, grpSelMaxIT, errBnd, thresholdingWay,...
%                         Nu, rem_ctrl, output);
%         GLasso_Biascorrt = true;
%         if GLasso_Biascorrt
%             Bs_GLasso_corrt = zeros(size(Bs_GLasso));
%             for paramInd = 1:size(nzPatts_GLasso, 1)
%                 tmpB = zeros(size(Bs_GLasso(:,:,paramInd)));
%                 tmpX = X(:, nzPatts_GLasso(paramInd, :)==1);
%                 tmpB2 = (tmpX' * tmpX) \ (tmpX' * Y);
%                 tmpB(nzPatts_GLasso(paramInd, :)==1, :) = tmpB2;
%                 Bs_GLasso_corrt(:, :, paramInd) = tmpB;
%             end    
%             % Select the best model according the validation error, and calc. its  test error        
%             [B_est, optInd, Berr, XBerr, tstErr] = validationAndTest_aux(Bs_GLasso_corrt, tuningMethod, lossType, X, Y, valX, valY, tstX, tstY, B_true, fixDesign, corrMat);
%         else
%             [B_est, optInd, Berr, XBerr, tstErr] = validationAndTest_aux(Bs_GLasso, tuningMethod, lossType, X, Y, valX, valY, tstX, tstY, B_true, fixDesign, corrMat);
%         end
%         
%         methodInd = methodInd + 1;
%         Berrs(methodInd, simuIndex) = Berr;
%         XBerrs(methodInd, simuIndex) = XBerr;
%         tsterrs(methodInd, simuIndex) = tstErr;
%         
%         % Check some info.
%         B_GLasso_opt = B_est;  grpB_GLasso_opt = grpBs_GLasso(optInd);
%         disp(['For GLasso solution path, the best estimate is obtained with nz rows=', num2str(grpB_GLasso_opt), ', from ', num2str(numel(gridLs_GLasso)), ' solutions, optInd=', num2str(optInd)])
%         GLassoModelsizes(:, simuIndex) = [grpB_GLasso_opt];
        
%         %% Method: RRR
%         rankGrid_RRR = [1:min([n, m, d])]; % ranksOfInterest;
%         [Bs_RRR, alphas_RRR, Ss_RRR, Vs_RRR, rankBs_RRR] = Func_RRR(X, Y, rankGrid_RRR, estimateIntercept);
%         % Select the best model according the validation error, and calc. its test error      
%         [B_est, optInd, Berr, XBerr, tstErr] = validationAndTest_aux(Bs_RRR, tuningMethod, lossType, X, Y, valX, valY, tstX, tstY, B_true, fixDesign, corrMat);
%         
%         methodInd = methodInd + 1;
%         Berrs(methodInd, simuIndex) = Berr;
%         XBerrs(methodInd, simuIndex) = XBerr;
%         tsterrs(methodInd, simuIndex) = tstErr;
%         
%         % Check some info.
%         B_RRR_opt = B_est;  rankB_RRR_opt = rankBs_RRR(optInd); 
%         disp(['For RRR solution path, the best estimate is obtained with rank=', num2str(rankB_RRR_opt)])
%         RRRModelsizes(:, simuIndex) = [rankB_RRR_opt];
        
%         %% Method: LASSO 
%         %     (We use the built-in lasso function in Matlab  with intercept; the Matlab version cannot be too old.)
%         methodInd = methodInd + 1;
%         % Initialize errors for lasso since errors are accumulated for each column
%         Berrs(methodInd, simuIndex) = 0;
%         XBerrs(methodInd, simuIndex) = 0;
%         tsterrs(methodInd, simuIndex) = 0;
%         % Loop for each column of response to get the best estimate for each column of B
%         for j = 1:m
%             % Create a lambda grid for lasso same as group lasso (because the default provided by the package may not be good)
%             nPoints = 100;
%             uBnd = max(sqrt(sum((X'*Y)'.^2)));
%             lLogBnd = -10;
%             lambdas = 2.^[lLogBnd:(log2(uBnd)-lLogBnd)/nPoints:log2(uBnd)];
%             lambdas = lambdas(end:-1:1);
%             % Use jth column as response to get the estimate for jth column of B
%             warning('off') % turn off warning because when there's no intercept, it pops a warning no intercept every time (feel free to keep it on when debugging)
%             [Bs_lasso_j, fit_info] = lasso(X, Y(:, j), 'Intercept', logical(estimateIntercept), 'Lambda', lambdas);
%             warning('on')
%             % Change into 3D matrix to properly call valudationAndTest_aux function
%             Bs_lasso_j = reshape(Bs_lasso_j, [d, 1, size(Bs_lasso_j, 2)]);
%             % Refit
%             Lasso_Biascorrt = true;
%             if Lasso_Biascorrt
%                 lasso_sol_numbers = size(Bs_lasso_j, 3);
%                 Bs_lasso_j_corrt = zeros(size(Bs_lasso_j));
%                 for solInd = 1:lasso_sol_numbers
%                     tmpB = zeros(size(Bs_lasso_j(:,:,solInd)));
%                     tmpX = X(:, Bs_lasso_j(:,:,solInd)~=0);
%                     tmpB(Bs_lasso_j(:,:,solInd)~=0, :) = (tmpX' * tmpX) \ (tmpX' * Y(:, j));
%                     Bs_lasso_j_corrt(:, :, solInd) = tmpB;
%                 end
%                 % Select the best model according the validation error, and calc. its test error
%                 [B_est_j, optInd_j, Berr_j, XBerr_j, tstErr_j] = validationAndTest_aux(Bs_lasso_j_corrt, tuningMethod, lossType, X, Y(:, j),...
%                     valX, valY(:, j), tstX, tstY(:, j), B_true(:, j), fixDesign, corrMat);
%             else
%                 % Select the best model according the validation error, and calc. its test error
%                 [B_est_j, optInd_j, Berr_j, XBerr_j, tstErr_j] = validationAndTest_aux(Bs_lasso_j, tuningMethod, lossType, X, Y(:, j),...
%                     valX, valY(:, j), tstX, tstY(:, j), B_true(:, j), fixDesign, corrMat);
%             end
%             Berrs(methodInd, simuIndex) = Berrs(methodInd, simuIndex) + Berr_j;
%             XBerrs(methodInd, simuIndex) = XBerrs(methodInd, simuIndex) + XBerr_j;
%             tsterrs(methodInd, simuIndex) = tsterrs(methodInd, simuIndex) + tstErr_j;
%         end
end

% % disp(['The mean errors for different methods'])
% % disp('B errors: '), mean(Berrs, 2)'
% % disp('XB errors: '), mean(XBerrs, 2)'
% % disp('Test errors: '), mean(tsterrs, 2)'

tsterrs = 2 * tsterrs; % We used half quadratic loss. Now change it to squared error.

disp(['The median errors for different methods'])
disp('B errors: '), median(Berrs, 2)'
disp('XB errors: '), median(XBerrs, 2)'
disp('Test errors: '), median(tsterrs, 2)'

disp(['CRL model size (median): ', num2str(median(CRLModelsizes, 2)')])
% disp(['GLasso model size (median): ', num2str(median(GLassoModelsizes, 2)')])
% disp(['RRR model size (median): ', num2str(median(RRRModelsizes, 2)')])


%% Write down the results
% Error_type_matrix = {'B errors: '; 'XB errors: '; 'Test errors: '};
% Error_matrix_mean = [ mean(Berrs, 2)'; mean(XBerrs, 2)'; mean(tsterrs, 2)'];
% Error_matrix_median = [ median(Berrs, 2)'; median(XBerrs, 2)'; median(tsterrs, 2)'];
% 
% file_id = fopen( 'simulation_methods_comparison.txt', 'w' );
% fprintf(file_id, 'The mean errors for different methods\n');
% for j = 1 : length( Error_type_matrix )
%     fprintf( file_id, '%s%.2f,  %.2f,  %.2f  %.2f\n', Error_type_matrix{j}, Error_matrix_mean(j,:) );
% end
% fprintf(file_id, '\nThe median errors for different methods\n');
% for j = 1 : length( Error_type_matrix )
%     fprintf( file_id, '%s%.2f,  %.2f,  %.2f  %.2f\n', Error_type_matrix{j}, Error_matrix_median(j,:) );
% end
% fprintf(file_id, '\n');
% fclose( file_id );

%% Method: Fused LASSO (call from R script: fused_lasso_simulation)
% % you need to have R installed in the environment to run this part;
% % Make sure you saving data if not done so before
% % Check the log if you cannot obtain an output file for fused lasso and lasso
% % You have to change simulation time manually in the R file to match the one you set here
% Rpath = 'C:\Program Files\R\R-3.6.1\bin'; % Important: Specify of YOUR R path here!!!
% RscriptFileName = './aux_functions/fused_lasso_simulation.R';  % Specify the R code address; note that a log file will be generated in this folder
% RunRcode(RscriptFileName, Rpath);



%% Plot for the results
% misspec_sigma = [0, 0.01, 0.02, 0.04, 0.08, 0.12]; 
% 
% B_errors_CRL = [1.6631 1.7984 2.1938 3.7439 9.9207 20.221];
% XB_errors_CRL = [1.5713 1.7002 2.0875 3.6023 9.7212 19.562];
% Test_errors_CRL = [13.313 13.372 13.567 14.318 17.365 22.251];
% 
% B_errors_Glasso = [23.0556 23.0713 23.1092 23.2547 23.8874 25.0098];
% XB_errors_Glasso = [21.2688 21.2614 21.2795 21.4399 22.0545 23.3870];
% Test_errors_Glasso = [23.0556 23.0594 23.0751 23.1598 23.4937 24.1952];
% 
% B_errors_RRR = [6.3435 6.4427 6.7463 7.9668 12.93 21.952];
% XB_errors_RRR = [5.9201 6.0157 6.312 7.5071 12.366 20.114];
% Test_errors_RRR = [15.518 15.567 15.718 16.323 18.72 22.57];
% 
% B_errors_lasso = [22.1758 22.1750 22.1399 22.1612 22.6161 23.1028];
% XB_errors_lasso = [20.5733 20.5525 20.5277 20.5995 21.0654 21.5911];
% Test_errors_lasso = [22.7563 22.7508 22.7547 22.8278 23.0267 23.3168];
% 
% B_errors_fusedlasso = [37.0735 38.559 37.459 39.0238 37.287 37.560];
% XB_errors_fusedlasso = [34.830 35.4819 35.112 36.423 34.570 35.024];
% Test_errors_fusedlasso = [31.196 31.898 31.309 32.079 31.369 31.446];
% 
% 
% figure(1);
% plot(misspec_sigma, B_errors_CRL, '-.+', 'MarkerSize',8, 'LineWidth',0.2, 'Color' , [1 0 0])
% hold on
% plot(misspec_sigma, B_errors_Glasso, '--s', 'MarkerSize',8, 'LineWidth',0.2, 'Color' , [0.4940 0.1840 0.5560])
% hold on
% plot(misspec_sigma, B_errors_RRR, '-.*', 'MarkerSize',8, 'LineWidth',0.2, 'Color' , [0.5 0.5 0])
% hold on
% plot(misspec_sigma, B_errors_lasso, '--x', 'MarkerSize',8, 'LineWidth',0.2, 'Color' , [0 0.4470 0.7410])
% hold on
% plot(misspec_sigma, B_errors_fusedlasso, '--x', 'MarkerSize',8, 'LineWidth',0.2, 'Color' , [0    0.3984    0.1992])
% xlim([0 0.12])
% xticks(misspec_sigma)
% ylim([1 60])
% yticks([1, 10, 30, 50])
% xlabel('sigma for misspecification')
% ylabel('Errors on B')
% legend('CRL','Group Lasso','RRR','Lasso','fused Lasso','Location','northwest')
% 
% 
% 
% figure(2);
% plot(misspec_sigma, Test_errors_CRL, '-.+', 'MarkerSize',8, 'LineWidth',0.2, 'Color' , [1 0 0])
% hold on
% plot(misspec_sigma, Test_errors_Glasso, '--s', 'MarkerSize',8, 'LineWidth',0.2, 'Color' , [0.4940 0.1840 0.5560])
% hold on
% plot(misspec_sigma, Test_errors_RRR, '-.*', 'MarkerSize',8, 'LineWidth',0.2, 'Color' , [0.5 0.5 0])
% hold on
% plot(misspec_sigma, Test_errors_lasso, '--x', 'MarkerSize',8, 'LineWidth',0.2, 'Color' , [0 0.4470 0.7410])
% hold on
% plot(misspec_sigma, Test_errors_fusedlasso, '--x', 'MarkerSize',8, 'LineWidth',0.2, 'Color' , [0    0.3984    0.1992])
% xlim([0 0.12])
% xticks(misspec_sigma)
% ylim([10 90])
% yticks([10, 30, 50, 70, 90])
% xlabel('sigma for misspecification')
% ylabel('Prediction Errors')
% legend('CRL','Group Lasso','RRR','Lasso','fused Lasso','Location','northwest')


function [B_est, optInd, Berr, XBerr, tstErr] = validationAndTest_aux(Bs_forTuning, tuningMethod, lossType, X, Y, ...
    valX, valY, tstX, tstY, B_true, fixDesign, corrMat)
% Given a solution path Bs_forTuning (3-dim array), we use parameter tuning
% to find the best estimate, and the evaluate its statistical performance

if contains(lossType, 'tracemodel')
    error('Matrix model Y ~ X * B is assumed. Not applicable to trace model.')
end
% use the validation dataset to find the best estimate 
if strcmp(tuningMethod, 'valData')
    valerrs = - ones(size(Bs_forTuning, 3), 1);
    for estInd = 1:size(Bs_forTuning, 3)
        B_est_tmp = Bs_forTuning(:, :, estInd);
        valerrs(estInd) = Func_Loss_Grad(valY, valX * B_est_tmp, lossType, 0) / size(valY, 1);
    end
else
    error('Not implemented yet') %CV, BIC, etc.
end
optInd = find(valerrs == min(valerrs), 1, 'first'); % valerrs(optInd)
B_est = Bs_forTuning(:, :, optInd);

% Evaluate test error
Berr = norm(B_true - B_est, 'fro')^2;
if fixDesign
    XBerr = norm(X * (B_true - B_est), 'fro')^2 / size(X, 1);
else
    XBerr = trace((B_true - B_est)' * corrMat * (B_true - B_est));
end
tstErr = Func_Loss_Grad(tstY, tstX * B_est, lossType, 0)/size(tstY, 1); % half-quadratic loss for 'gaussian'
end
